<?php

/**
 * Class Gene_Braintree_Model_Observer
 *
 * @author Dave Macaulay <dave@gene.co.uk>
 */
class Gene_Braintree_Model_Observer
{

    /**
     * Detect which checkout is in use and add a new layout handle
     *
     * @param Varien_Event_Observer $observer
     *
     * @return $this
     */
    public function addLayoutHandle(Varien_Event_Observer $observer)
    {
        /* @var $action Mage_Core_Controller_Varien_Action */
        $action = $observer->getAction();

        /* @var $layout Mage_Core_Model_Layout */
        $layout = $observer->getLayout();

        // We only want to run this action on the checkout
        if($action->getFullActionName() == 'checkout_onepage_index') {

            // Attempt to detect Amasty_Scheckout
            if (Mage::helper('core')->isModuleEnabled('Amasty_Scheckout')) {
                $layout->getUpdate()->addHandle('amasty_onestep_checkout');
            }

            // Attempt to detect Unicode OP Checkout
            if (Mage::helper('core')->isModuleEnabled('Uni_Opcheckout') && Mage::helper('opcheckout')->isActive()) {
                $layout->getUpdate()->addHandle('unicode_onestep_checkout');
            }

            // Detect the Oye one step checkout
            if(Mage::helper('core')->isModuleEnabled('Oye_Checkout') && Mage::helper('oyecheckout')->isOneStepLayout()) {
                $layout->getUpdate()->addHandle('oye_onestep_checkout');
            }
        }

        // As some 3rd party checkouts use the same handles, and URL we have to dynamically add new handles
        if($action->getFullActionName() == 'onestepcheckout_index_index') {

            // Attempt to detect Magestore_Onestepcheckout
            if (Mage::helper('core')->isModuleEnabled('Magestore_Onestepcheckout')) {
                if(Mage::helper('onestepcheckout')->enabledOnestepcheckout()) {
                    $layout->getUpdate()->addHandle('magestore_onestepcheckout_index');
                }
            }

            // Attempt to detect Idev_OneStepCheckout
            if (Mage::helper('core')->isModuleEnabled('Idev_OneStepCheckout')) {
                $layout->getUpdate()->addHandle('idev_onestepcheckout_index');
            }
        }

        return $this;
    }

    /**
     * Store the generated customer ID if it's present in the session
     *
     * @param \Varien_Event_Observer $observer
     *
     * @return $this
     */
    public function completeCheckout(Varien_Event_Observer $observer)
    {
        // Do we have a customer ID within the session?
        if(Mage::getSingleton('checkout/session')->getBraintreeCustomerId() &&
            Mage::getSingleton('checkout/type_onepage')->getCheckoutMethod() == Mage_Checkout_Model_Type_Onepage::METHOD_CUSTOMER) {

            // Get the customer
            $customer = Mage::getSingleton('customer/session')->getCustomer();

            // Save the Braintree customer ID
            $customer->setBraintreeCustomerId(Mage::getSingleton('checkout/session')->getBraintreeCustomerId())->save();
        }

        // Perform any cleaning up required
        Gene_Braintree_Model_Wrapper_Braintree::cleanUp();

        // Unset the ID from the session
        Mage::getSingleton('checkout/session')->unsetData('braintree_customer_id');

        return $this;
    }

    /**
     * Capture payment on shipment if set
     *
     * @param Varien_Event_Observer $observer
     *
     * @return $this
     */
    public function captureBraintreePayment(Varien_Event_Observer $observer)
    {
        /* @var $shipment Mage_Sales_Model_Order_Shipment */
        $shipment = $observer->getEvent()->getShipment();

        /* @var $order Mage_Sales_Model_Order */
        $order = $shipment->getOrder();

        // Should we capture the payment in shipment?
        if($this->shouldCaptureShipment($order)) {

            // Check the order can be invoiced
            if($order->canInvoice()) {

                /* @var @invoice Mage_Sales_Model_Order_Invoice */
                $invoice = Mage::getModel('sales/service_order', $order)->prepareInvoice();

                // Check the invoice has items to invoice
                if ($invoice->getTotalQty()) {

                    // Set the requested capture case
                    $invoice->setRequestedCaptureCase(Mage_Sales_Model_Order_Invoice::CAPTURE_ONLINE);

                    // Register the invoice
                    $invoice->register();

                    // Save the transaction
                    $transactionSave = Mage::getModel('core/resource_transaction')
                        ->addObject($invoice)
                        ->addObject($invoice->getOrder());

                    // Save the transaction
                    $transactionSave->save();

                }

            }

        }

        return $this;
    }

    /**
     * Should we capture the payment?
     *
     * @param \Mage_Sales_Model_Order $order
     *
     * @return bool
     */
    private function shouldCaptureShipment($order)
    {
        // Check the store configuration settings are set to capture shipment
        if(Mage::getStoreConfig(Gene_Braintree_Model_Source_Creditcard_PaymentAction::PAYMENT_ACTION_XML_PATH, $order->getStoreId()) == Mage_Payment_Model_Method_Abstract::ACTION_AUTHORIZE
            && Mage::getStoreConfig(Gene_Braintree_Model_Source_Creditcard_CaptureAction::CAPTURE_ACTION_XML_PATH, $order->getStoreId()) == Gene_Braintree_Model_Source_Creditcard_CaptureAction::CAPTURE_SHIPMENT)
        {
            return true;
        }
        return false;
    }

    /**
     * Check if compilation is enabled, if so copy over the certificates
     *
     * @return $this
     */
    public function checkCompilation()
    {
        // Determine whether the compiler has been enabled
        if(defined('COMPILER_INCLUDE_PATH')) {
            $certificates = array('api_braintreegateway_com.ca.crt', 'sandbox_braintreegateway_com.ca.crt');
            $compilerPath = COMPILER_INCLUDE_PATH;
            $directory = '/ssl/';

            // Verify the SSL folder exists
            if(!is_dir($compilerPath . '/..' . $directory)) {
                mkdir($compilerPath . '/..' . $directory);
            }

            // Loop through each certificate and check whether it's in the includes directory, if not copy it!
            foreach($certificates as $file) {
                if(!file_exists($compilerPath . '/..' . $directory . $file)) {
                    copy(Mage::getBaseDir('lib') . $directory . $file, $compilerPath . '/..' . $directory . $file);
                }
            }
        }

        return $this;
    }
}